package eu.etaxonomy.taxeditor.prototype2.model;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.databinding.observable.value.IObservableValue;

/**
 * 
 * 
 * @author p.ciardelli
 *
 */
public class PropertySheetNode {
	
	private boolean editable;
	private List<PropertySheetNode> childProperties;
	private PropertySheetNode parentProperty;
	private String propertyField;
	private String propertyValue;
	private IObservableValue observeValue;
	
	public static boolean EDITABLE = true;
	public static boolean NOT_EDITABLE = false;
	
	public PropertySheetNode(String field, String value, 
			PropertySheetNode parent, boolean editable) {
		this.propertyField = field;
		this.propertyValue = value;
		this.parentProperty = parent;
		if (parent != null) parent.addChildProperty(this);
		this.editable = editable;
	}
	
	public PropertySheetNode(String field, IObservableValue observeValue, 
			PropertySheetNode parent, boolean editable) {
		this(field, (String) observeValue.getValue(), parent, editable);
		this.observeValue = observeValue;
	}
	
	private void addChildProperty(PropertySheetNode childProperty ) {
		if (childProperties == null)
			childProperties = new ArrayList<PropertySheetNode>();
		if (!childProperties.contains(childProperty))
			childProperties.add(childProperty);
	}

	public boolean isEditable() {
		return editable;
	}

	public void setEditable(boolean editable) {
		this.editable = editable;
	}

	public String getPropertyField() {
		return propertyField;
	}

	public void setPropertyField(String propertyField) {
		this.propertyField = propertyField;
	}

	public String getPropertyValue() {
		if (observeValue != null)
			return (String) observeValue.getValue();
		return propertyValue;
	}

	public void setPropertyValue(String propertyValue) {
		this.propertyValue = propertyValue;
	}

	public List<PropertySheetNode> getChildProperties() {
		return childProperties;
	}

	public void setChildProperties(List<PropertySheetNode> subProperties) {
		this.childProperties = subProperties;
	}

	public PropertySheetNode getParentProperty() {
		return parentProperty;
	}

	public void setParentProperty(PropertySheetNode parentProperty) {
		this.parentProperty = parentProperty;
	}	
	
	public IObservableValue getObserveValue() {
		return this.observeValue;
	}
}
