package eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport;

import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.rules.IPartitionTokenScanner;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.Token;


public class NamePartitionScanner implements IPartitionTokenScanner {
	
	public final static String GENUS = "genus";
	public final static String SPECIFIC_EPITHET = "specificEpithet";
	public final static String EVERYTHING_ELSE = "everythingElse";
	
	public final static String[] PARTITION_TYPES = {GENUS, SPECIFIC_EPITHET, EVERYTHING_ELSE};
	
	private NameDocument document;
	private int offset;
	private int currentOffset;
	private int length;
	private TaxonName taxonName;
		
	String [] words;
	int index;
	private String[] words2;
	
	private String text;
	
	public NamePartitionScanner() {
	}
	
	public void setPartialRange(IDocument document, int offset, int length,
			String contentType, int partitionOffset) {

		System.out.println("--->\t" + offset + "\n\t" + length
				 + "\n\t" + contentType + "\n\t" + partitionOffset);
		
		Assert.isTrue(document instanceof NameDocument, "NamePartitionScanner can only scan a NameDocument.");
		this.document = (NameDocument) document;

		text = document.get();
				
		// check whether this is the initial scan
		if (this.taxonName == null) {
			
			this.taxonName = this.document.getTaxonName();
			
			// is this a pre-existing taxon name? 
			if (this.taxonName.getGenus() != null) {
				
				// map all partition types to their respective taxon name properties
				words = new String [] {this.taxonName.getGenus(), this.taxonName.getSpecificEpithet()};
				this.document.set(this.taxonName.getGenus() + " " + this.taxonName.getSpecificEpithet());
				
			}
		} else if (text.length() > 0) {
			
			// does text contain line breaks?
//			if (text.contains(System.getProperty("line.separator"))) {
//				Scanner scanner = new Scanner(text);
//				scanner.useDelimiter (System.getProperty("line.separator"));
//				
////				document.set(scanner.next());
//				
//				while (scanner.hasNext()) {
//					System.out.println("Create new relname with: " + scanner.next());
//				}
//			}
			
			// only scan first line if there are line breaks in the text
//			Scanner scanner = new Scanner(text);
//			scanner.useDelimiter (System.getProperty("line.separator"));
//			text = scanner.next();
			// TODO replace with regexp pattern matching
			words = text.split("\\s");
		}
		
//		System.out.println("setPartialRange:\n\t" + offset + "\n\t" + length + "\n\t" + contentType + "\n\t" + partitionOffset);

		this.index = 0;
		this.currentOffset = 0;
	}

	public int getTokenLength() {		
		return this.length;
	}

	public int getTokenOffset() {
		return this.offset;
	}

	public IToken nextToken() {
		IToken token;
		if (index == words.length || index == PARTITION_TYPES.length) {
			token = Token.EOF;
			System.out.println("EOFMOFO" + text);
		} else {
			token = new Token(PARTITION_TYPES[index]);
			this.length = words[index].length();
			this.offset = this.currentOffset;
			
		}
		
		index++;
		currentOffset += this.length + 1; 
			
		return token;
	}

	public void setRange(IDocument document, int offset, int  length) {
		setPartialRange(document, offset, length, null, -1);
	}
}