package eu.etaxonomy.taxeditor.prototype2.view.propertysheetsupport;

import java.util.TreeMap;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.UpdateValueStrategy;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;

import eu.etaxonomy.taxeditor.prototype2.model.PropertySheetNode;
import eu.etaxonomy.taxeditor.prototype2.view.PropertySheetViewer;

public class PropertySheetValueEditingSupport extends EditingSupport {

	private TreeMap<String,CellEditor> cellEditors;
	private PropertySheetViewer viewer;
	private DataBindingContext bindingContext;
			
	public PropertySheetValueEditingSupport(PropertySheetViewer viewer) {
		super(viewer);
		this.viewer = viewer;
		this.bindingContext = viewer.getBindingContext();
		this.cellEditors = new TreeMap<String, CellEditor>();
	}		
	
	protected boolean canEdit(Object element) {
		return ((PropertySheetNode) element).isEditable();
	}

	protected CellEditor getCellEditor(Object element) {
		String elementString = element.toString();
		if (cellEditors.get(elementString) == null)
			cellEditors.put(elementString, new TextCellEditor(viewer.getTree()));
		return cellEditors.get(elementString);
	}

	protected Object getValue(Object element) {
		return (PropertySheetNode) element;
	}

	protected void setValue(Object element, Object value) {
		((PropertySheetNode) element).setPropertyValue(value.toString());
		this.getViewer().update(element, null);
	}

	protected void initializeCellEditorValue(CellEditor cellEditor, ViewerCell cell) {
		PropertySheetNode node = (PropertySheetNode) getValue(cell.getElement());
		
		// an empty string instead of a null makes the field editable
		cellEditor.setValue(node.getPropertyValue() == null ? "" : node.getPropertyValue());
		
		// if node element is observing a value, bind it to its TreeItem
		IObservableValue observeNodeValue = node.getObserveValue();
		if (observeNodeValue != null) {
			
	        Assert.isNotNull(bindingContext,
            	"Editing support: PropertySheetViewer's binding context must be explicitly set.");
			
			IObservableValue observeCellValue = SWTObservables.observeText(cellEditor.getControl(), SWT.Modify);
			bindingContext.bindValue(observeNodeValue, observeCellValue,
					new UpdateValueStrategy(UpdateValueStrategy.POLICY_CONVERT), null);
			
		}
	}		
}