/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.actions.name;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author p.ciardelli
 * @created 09.10.2008
 * @version 1.0
 */
public class Messages {
	private static final Logger logger = Logger.getLogger(Messages.class);
	private static final String BUNDLE_NAME = "eu.etaxonomy.taxeditor.actions.name.messages"; //$NON-NLS-1$

	private static final ResourceBundle RESOURCE_BUNDLE = ResourceBundle
			.getBundle(BUNDLE_NAME, Locale.getDefault());

	private Messages() {
	}

	public static String getString(String key) {
		try {
			return RESOURCE_BUNDLE.getString(PreferencesUtil.concatCodeMessageSuffix(key));
		} catch (MissingResourceException e1) {
			try {
				return RESOURCE_BUNDLE.getString(key);
			} catch (MissingResourceException e2) {
				return '!' + key + '!';
			}	
		}
	}

	public static String getString(String key, TaxonNameBase name) {
		if (name instanceof BotanicalName) {
			key += "." + ITaxEditorConstants.CODE_PREFERENCE_ICBN;
		} else if (name instanceof ZoologicalName) {
			key += "." + ITaxEditorConstants.CODE_PREFERENCE_ICZN;
		} else {
			getString(key);
		}
		try {
			return RESOURCE_BUNDLE.getString(key);
		} catch (MissingResourceException e2) {
			return '!' + key + '!';
		}
	}
}
