/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.springframework.transaction.TransactionStatus;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.actions.SaveAllAction;
import eu.etaxonomy.taxeditor.actions.cdm.SaveTaxonAction;
import eu.etaxonomy.taxeditor.editor.description.TaxonDescriptionEditor;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.model.CdmUtil;
import eu.etaxonomy.taxeditor.navigation.RecentNamesView;

/**
 * 
 * Generates the tabbed editor with <code>TaxonNameEditor</code> on top and tabs for
 *  "Desriptions", "Concepts", "Geography", etc.
 * 
 * @author p.ciardelli
 * @created 15.05.2008
 * @version 1.0
 */
public class MultiPageTaxonEditor extends MultiPageEditorPart {
	private static final Logger logger = Logger.getLogger(MultiPageTaxonEditor.class);

	public static final String ID = "eu.etaxonomy.taxeditor.editor.multipagetaxonview";
	
	private Taxon taxon;
	private boolean dirty;

	private TransactionStatus tx;
		
	public MultiPageTaxonEditor() {
		super();
//		tx = CdmUtil.startTransaction();
	}
	
	@Override
	public void dispose() {
		super.dispose();
//		CdmUtil.commitTransaction(tx);
	}
	
	@Override
	protected void createPages() {
				
		try {
			addPage(0, new TaxonNameEditor(), getEditorInput());
			setPageText(0, "Name");
			
			// TODO lazy create
			addPage(1, new TaxonDescriptionEditor(), getEditorInput());
			setPageText(1, "Descriptive");
			
//			addPage(2, new EmptyEditorView(), getEditorInput());
//			setPageText(2, "Concepts");
//			
//			addPage(3, new EmptyEditorView(), getEditorInput());
//			setPageText(3, "Geographic");
					
		} catch (PartInitException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public void doSave(IProgressMonitor monitor) {
		
		// Bummer quick fix - saving individual taxa leads to too many
		//	concurrency errors
		new SaveAllAction().run(null);
		
		// Show new taxon being saved for the first time in recent names
		RecentNamesView.addRecentName(taxon);
		
////		monitor.beginTask(name, totalWork)
//		new SaveTaxonAction(taxon).run();
////		CdmUtil.commitTransaction(tx);
//		setDirty(false);
////		tx = CdmUtil.startTransaction();
////		CdmUtil.getTaxonService().saveTaxon(taxon);
	}

	private void setDirty(boolean dirty) {
		this.dirty = dirty;
		firePropertyChange(PROP_DIRTY);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.MultiPageEditorPart#isDirty()
	 */
	public boolean isDirty() {
		return dirty;
	}

    /**
     * Checks whether nested editors are calling <code>firePropertyChange(PROP_DIRTY)</code>
     * to signal an edit has taken place before passing property change along to 
     * <code>super.handlePropertyChange(int propertyId)</code>.
     */
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.MultiPageEditorPart#handlePropertyChange(int)
	 */
	protected void handlePropertyChange(int propertyId) {
		if (propertyId == PROP_DIRTY) {
			setDirty(true);
		}
		super.handlePropertyChange(propertyId);
	}
	
	@Override
	public void doSaveAs() {}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	@Override
	public void init(IEditorSite site, IEditorInput input) throws PartInitException {
		
		if (!(input instanceof IEditorInput))
			throw new PartInitException(
					"Invalid Input: Must be IEditorInput");
		
		// Get taxon from editor input
		if (input.getAdapter(Taxon.class) != null) {
			taxon = (Taxon) input.getAdapter(Taxon.class);
		} else {
			taxon = null;
		}

		// Save taxon to put it into current transaction
		CdmUtil.getTaxonService().saveTaxon(taxon);
		
		// Listen for name changes, 
		//  change tab for this taxon editor accordingly
		taxon.addPropertyChangeListener("name", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				setPartName();
			}
		});

		setPartName();
		
		super.init(site, input);
	}
		
    /**
     * Calls <code>MultiPageEditorPart.setPartName(String partName)</code>
     *  with text appropriate to the state of the taxon: any taxon that has 
     *  been saved will by necessity have a name to display; a new taxon 
     *  should display "New taxon" in the editor tab.
     */
    private void setPartName() {
        
    	String partName = null;
    	TaxonNameBase name = taxon.getName();
    	
    	if (name != null) {
    		partName = name.getTitleCache();
    	}
    	
    	if (partName == null || partName.equals("")) {
    		partName = ("New taxon");
    	}

        setPartName(partName);
    }
    
    /**
     * SaveAllAction needs to set this.
     * 
     * @param IS_DIRTY
     */
    public void setDirtyExtern(boolean IS_DIRTY) {
    	setDirty(IS_DIRTY);
    }
    
    public Taxon getTaxon(){
    	return this.taxon;
    }
}