/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.actions.name.RemoveConceptAction;
import eu.etaxonomy.taxeditor.controller.EditorController;
import eu.etaxonomy.taxeditor.controller.GlobalController;
import eu.etaxonomy.taxeditor.editor.ContextMenu;
import eu.etaxonomy.taxeditor.operations.name.ChangeMisappliedNameToSynonymOperation;
import eu.etaxonomy.taxeditor.propertysheet.name.TaxonBasePropertySource;

/**
 * @author p.ciardelli
 * @created 13.01.2009
 * @version 1.0
 */
public class MisappliedNameComposite extends NameComposite {
	private static final Logger logger = Logger
			.getLogger(MisappliedNameComposite.class);
	
	private Taxon misappliedName;
	
	public MisappliedNameComposite(Composite parent, IManagedForm form,
			Taxon taxon, Taxon misappliedName) {
		super(parent, form, NameComposite.MISAPPLIED_NAME, misappliedName);
		
		this.taxon = taxon;
		this.misappliedName = misappliedName;
		
		setIsDraggable(true);
		setIcon(MISAPPLIEDNAME_ICON);
		setFont(getViewerFont());
		setIndent(MISAPPLIEDNAME_INDENT);

		createMenu();
		
		showSec();		
		
		initTextViewer(misappliedName);
	}

	private void showSec() {
		if (misappliedName == null) {
			return;
		}
		
		if (misappliedName.getSec() == null) {
			setNonEditableInfo("sec. ???");
		} else {
			setNonEditableInfo("sec. " + misappliedName.getSec().getTitleCache());
		}
	}
	
	public Taxon getMisappliedName() {
		return misappliedName;
	}
	
	private void createMenu() {

		ContextMenu contextMenu = createContextMenu();

		// Remove misapplied name from taxon
		contextMenu.addAction(new RemoveConceptAction(taxon, 
				misappliedName, TaxonRelationshipType.MISAPPLIED_NAME_FOR()));

	}
	
	public boolean setParent(Composite parent) {

		if (super.setParent(parent)) {	
			
			// If this has been moved to the misapplied names group, do nothing
			if (parent instanceof MisappliedGroupComposite) {
				return true;
			}

			// Has this been moved to a HomotypicalGroup?
			if (parent instanceof HomotypicalGroupComposite) {
				
				HomotypicalGroup homotypicalGroup = 
						((HomotypicalGroupComposite)parent).getGroup();
				
				IUndoContext undoContext = EditorController.getUndoContext(taxon);
				IUndoableOperation operation = new ChangeMisappliedNameToSynonymOperation
						("change misapplication to synonym", undoContext, taxon, misappliedName, homotypicalGroup);
				
				GlobalController.executeOperation(operation);
			}
			return true;

		}
		return false;
	}
	
	public IPropertySource getPropertySource() {
		return new TaxonBasePropertySource(misappliedName, "Misapplied Name");
	}

	@Override
	protected Font getViewerFont() {
		return MISAPPLIEDNAME_FONT;
	}
	
	protected void calculateErrors() {
		super.calculateErrors();
		
		textViewer.setShowSecError(misappliedName);
	}

	@Override
	protected TaxonNameBase getName() {
		return misappliedName.getName();
	}
	
	public Object getData () {
		return getMisappliedName();
	}
}